"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var extractor_utils_1 = require("../extractor-utils");
var types_1 = require("@babel/types");
var NOT_WEBPACK_BOOTSTRAP_AST_ERR = new TypeError("not a webpackBootstrap function call AST.");
var moduleFunctionParams = ["module", "exports", "__webpack_require__"];
exports.extractModules = function (ast, options) {
    var e_1, _a;
    ast = extractor_utils_1.getInputProgramAST(ast);
    if (Array.isArray(options.bundleAstReferenceKeys)) {
        try {
            for (var _b = __values(options.bundleAstReferenceKeys), _c = _b.next(); !_c.done; _c = _b.next()) {
                var k = _c.value;
                ast = ast[k];
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    else {
        throw new TypeError("options.moduleAstEntryKeys is not an array.");
    }
    if (!types_1.isCallExpression(ast)) {
        throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
    }
    var _d = ast, callee = _d.callee, fArgument = _d.arguments[0];
    if (!types_1.isFunctionExpression(callee)) {
        throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
    }
    // try to get entry id
    if (typeof options.entryPoint !== "string" || typeof options.entryPoint !== "number") {
        options.entryPoint = undefined;
        try {
            // unminified file
            // the last statement:
            // e.g. `return __webpack_require__(__webpack_require__.s = 8);`
            // minified file
            // the last statement:
            // e.g. `return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=8)`
            var lastStatement = callee.body.body.pop();
            if (!types_1.isReturnStatement(lastStatement)) {
                throw new Error();
            }
            var e = lastStatement.argument;
            var entryRequireCall = void 0;
            if (types_1.isCallExpression(e)) {
                entryRequireCall = e;
            }
            else if (types_1.isSequenceExpression(e)) {
                entryRequireCall = e.expressions.pop();
            }
            if (!types_1.isCallExpression(entryRequireCall)) {
                throw new Error();
            }
            var entryRequireCallArgument = entryRequireCall.arguments[0];
            if (types_1.isNumericLiteral(entryRequireCallArgument)) {
                options.entryPoint = entryRequireCallArgument.value;
            }
            else if (types_1.isAssignmentExpression(entryRequireCallArgument)) {
                var a = entryRequireCallArgument.right;
                if (types_1.isNumericLiteral(a)) {
                    options.entryPoint = a.value;
                }
            }
            if (typeof options.entryPoint == "undefined") {
                throw new Error();
            }
        }
        catch (_) {
            throw new Error("options.entryPoint is undefined and failed to get entry id.");
        }
    }
    var entryID = options.entryPoint;
    var entryPath = "entry_" + entryID + ".ts";
    var modules = {};
    var solveModule = function (moduleFunction, id) {
        var isEntry = id == entryID;
        // get module ast block
        if (!types_1.isFunctionExpression(moduleFunction)) {
            throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
        }
        var params = moduleFunction.params, moduleAST = moduleFunction.body;
        var modulePath = "" + (isEntry ? "entry_" : "") + id + ".ts";
        if (!options.replaceModuleFunctionParams) {
            // put module function params in module body 
            // minified `function(e,t,n){`
            // unminified `function(module, exports, __webpack_require__) {`
            // -> `const e = module; const t = exports; const n = __webpack_require__;`
            params.forEach(function (p, index) {
                if (!types_1.isIdentifier(p)) {
                    throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
                }
                moduleAST.body.unshift(types_1.variableDeclaration("const", [types_1.variableDeclarator(types_1.identifier(p.name), types_1.identifier(moduleFunctionParams[index]))]));
            });
        }
        else {
            // TODO
        }
        modules[modulePath] = {
            id: id,
            ast: moduleAST,
            isEntry: isEntry,
        };
    };
    if (types_1.isArrayExpression(fArgument)) {
        fArgument.elements.forEach(solveModule);
    }
    else if (types_1.isObjectExpression(fArgument)) {
        fArgument.properties.forEach(function (p) {
            if (!types_1.isObjectProperty(p)) {
                throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
            }
            if (!types_1.isStringLiteral(p.key) && !types_1.isNumericLiteral(p.key)) {
                throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
            }
            if (!types_1.isFunctionExpression(p.value)) {
                throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
            }
            solveModule(p.value, p.key.value);
        });
    }
    else {
        throw NOT_WEBPACK_BOOTSTRAP_AST_ERR;
    }
    return {
        modules: modules,
        entry: options.entryPoint,
        entryPath: entryPath,
    };
};
exports.default = exports.extractModules;
