"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("@babel/types");
var visitor_wrapper_1 = __importDefault(require("../utils/visitor-wrapper"));
var is_void0_1 = __importDefault(require("../utils/is-void0"));
var EQUALITY_BINARY_OPERATORS = ["==", "===", "!=", "!=="];
var BOOLEAN_NUMBER_BINARY_OPERATORS = [">", "<", ">=", "<="];
/**
 * `null !== bar` -> `bar !== null`
 * @see https://babeljs.io/docs/en/babel-plugin-minify-flip-comparisons (reversed)
 */
exports.flipComparisons = visitor_wrapper_1.default({
    // modify from https://github.com/babel/minify/blob/master/packages/babel-plugin-minify-flip-comparisons/src/index.js
    BinaryExpression: function (path) {
        var node = path.node;
        var right = node.right, left = node.left;
        // Make sure we have a constant on the right.
        if (types_1.isLiteral(right) ||
            is_void0_1.default(right) ||
            (types_1.isUnaryExpression(right) && types_1.isLiteral(right.argument)) ||
            types_1.isObjectExpression(right) ||
            types_1.isArrayExpression(right)) {
            return;
        }
        // Commutative operators.
        if (EQUALITY_BINARY_OPERATORS.indexOf(node.operator) >= 0 ||
            ["*", "^", "&", "|"].indexOf(node.operator) >= 0) {
            node.left = right;
            node.right = left;
            return;
        }
        if (BOOLEAN_NUMBER_BINARY_OPERATORS.indexOf(node.operator) >= 0) {
            node.left = right;
            node.right = left;
            var operator = void 0;
            switch (node.operator) {
                case ">":
                    operator = "<";
                    break;
                case "<":
                    operator = ">";
                    break;
                case ">=":
                    operator = "<=";
                    break;
                case "<=":
                    operator = ">=";
                    break;
            }
            node.operator = operator;
            return;
        }
    },
});
exports.default = exports.flipComparisons;
